package com.fr.decision.auth.http;

import com.fanruan.api.decision.login.LoginKit;
import com.fanruan.api.log.LogKit;
import com.fanruan.api.net.http.HttpKit;
import com.fanruan.api.script.ScriptKit;
import com.fanruan.api.util.StringKit;
import com.fr.decision.auth.http.config.HttpAuthorizeConfig;
import com.fr.decision.fun.impl.AbstractHttpAuthorizeProvider;
import com.fr.intelli.record.Focus;
import com.fr.intelli.record.Original;
import com.fr.record.analyzer.EnableMetrics;

import javax.script.ScriptEngine;
import java.util.HashMap;
import java.util.Map;

/**
 * http认证更改内置的认证逻辑，可以通过脚本制定判断认证成功或者失败的策略
 */
@EnableMetrics
public class HttpAuthorizeBridge extends AbstractHttpAuthorizeProvider {

    private static final String SCRIPT_TPL = "(function(username, value) {return %s;})(\"%s\", \"%s\")";

    private ScriptEngine engine = ScriptKit.newScriptEngine();

    @Override
    public Scope scope() {
        return Scope.REPLACE;
    }

    @Override
    @Focus(id = "com.fr.decision.auth.http", text = "", source = Original.PLUGIN)
    public boolean authorize(String username, String inputPassword, String savedPassword, String hashPassword) {
        Map<String, String> queryMap = new HashMap<String, String>();
        queryMap.put("username", username);
        queryMap.put("inputPassword", inputPassword);
        String httpUrl = LoginKit.getHttpPassportUrl();
        if (StringKit.isEmpty(httpUrl)) {
            return false;
        }
        try {
            String resultText = HttpKit.get(httpUrl, queryMap);
            String condition = HttpAuthorizeConfig.getInstance().getCondition();
            if (StringKit.isBlank(condition)) {
                if (!"false".equals(resultText)) {
                    return true;
                }
            } else {
                Object r = engine.eval(String.format(SCRIPT_TPL, HttpAuthorizeConfig.getInstance().getCondition(), username, resultText));
                if (Boolean.TRUE == r) {
                    return true;
                }
            }
        } catch (Exception e) {
            LogKit.error(e.getMessage(), e);
        }
        return false;
    }

    public static void main(String... args) throws Exception {
        ScriptEngine scriptEngine = ScriptKit.newScriptEngine();
        Object r = scriptEngine.eval(String.format(SCRIPT_TPL, "value!=\"false\"", "alex", "false"));
        System.out.println("r=" + r);
    }
}
